using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;
using Microsoft.Surface.Presentation;
using Microsoft.Surface.Presentation.Controls;
using Microsoft.Surface.Presentation.Input;
using Blake.NUI.WPF.Touch;
using Blake.NUI.WPF.Utility;
using System.Collections.ObjectModel;

namespace SurfaceDragDropSample
{
    /// <summary>
    /// Interaction logic for SurfaceWindow1.xaml
    /// </summary>
    public partial class SurfaceWindow1 : SurfaceWindow
    {
        #region Fields

        Dictionary<SurfaceDragCursor, UIElement> CursorToSourceVisual = new Dictionary<SurfaceDragCursor, UIElement>();

        #endregion

        #region Properties

        #region FruitItems

        private ObservableCollection<FoodModel> _fruitItems;
        public ObservableCollection<FoodModel> FruitItems
        {
            get
            {
                if (_fruitItems == null)
                    _fruitItems = new ObservableCollection<FoodModel>();
                return _fruitItems;
            }
        }

        #endregion

        #region MixedItems

        private ObservableCollection<FoodModel> _mixedItems;
        public ObservableCollection<FoodModel> MixedItems
        {
            get
            {
                if (_mixedItems == null)
                    _mixedItems = new ObservableCollection<FoodModel>();
                return _mixedItems;
            }
        }

        #endregion

        #region VegetableItems

        private ObservableCollection<FoodModel> _vegetableItems;
        public ObservableCollection<FoodModel> VegetableItems
        {
            get
            {
                if (_vegetableItems == null)
                    _vegetableItems = new ObservableCollection<FoodModel>();
                return _vegetableItems;
            }
        }

        #endregion

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor.
        /// </summary>
        public SurfaceWindow1()
        {
            LoadData();
            this.DataContext = this;

            InitializeComponent();
            NativeTouchDevice.RegisterEvents(this);
        }

        #endregion

        private void LoadData()
        {
            MixedItems.Add(new FoodModel() { Name = "Apple", Type = FoodType.Fruit });
            MixedItems.Add(new FoodModel() { Name = "Broccoli", Type = FoodType.Vegetable });
            MixedItems.Add(new FoodModel() { Name = "Banana", Type = FoodType.Fruit });
            MixedItems.Add(new FoodModel() { Name = "Radish", Type = FoodType.Vegetable });
        }

        private void lbx_PreviewTouchDown(object sender, TouchEventArgs e)
        {
            FrameworkElement source = sender as FrameworkElement;
            var element = e.TouchDevice.DirectlyOver as FrameworkElement;
            var view = VisualUtility.FindVisualParent<FoodView>(element);
            if (view == null || view.DataContext == null)
                return;

            var data = view.DataContext;

            List<InputDevice> devices = new List<InputDevice>();
            devices.Add(e.TouchDevice);

            var cursorView = new FoodView();
            cursorView.DataContext = data;

            SurfaceDragCursor cursor = SurfaceDragDrop.BeginDragDrop(source, view, cursorView, data, devices, DragDropEffects.All);

            view.Visibility = System.Windows.Visibility.Hidden;
            CursorToSourceVisual.Add(cursor, view);
        }

        private void items_Drop(object sender, SurfaceDragDropEventArgs e)
        {
            ItemsControl target = sender as ItemsControl;
            var targetCollection = target.ItemsSource as ICollection<FoodModel>;

            var data = e.Cursor.Data as FoodModel;
            if (data != null)
                targetCollection.Add(data);
            e.Handled = true;
        }

        private void items_DragEnter(object sender, SurfaceDragDropEventArgs e)
        {
            ItemsControl target = sender as ItemsControl;
            var data = e.Cursor.Data as FoodModel;

            if (data == null ||
                e.Cursor.DragSource == target ||
                (data.Type == FoodType.Fruit && target.ItemsSource == VegetableItems) ||
                (data.Type == FoodType.Vegetable && target.ItemsSource == FruitItems))
            {
                e.Effects = DragDropEffects.None;
            }
            e.Handled = true;
        }

        private void items_DragLeave(object sender, SurfaceDragDropEventArgs e)
        {
            e.Effects = e.Cursor.AllowedEffects;
            e.Handled = true;
        }

        private void items_DragCanceled(object sender, SurfaceDragDropEventArgs e)
        {
            if (CursorToSourceVisual.ContainsKey(e.Cursor))
            {
                CursorToSourceVisual[e.Cursor].Visibility = System.Windows.Visibility.Visible;
                CursorToSourceVisual.Remove(e.Cursor);
            }
            e.Handled = true;
        }

        private void items_DragCompleted(object sender, SurfaceDragCompletedEventArgs e)
        {
            ItemsControl source = sender as ItemsControl;

            var data = e.Cursor.Data as FoodModel;
            if ((e.Cursor.Effects & DragDropEffects.Move) == DragDropEffects.Move)
            {
                var sourceCollection = source.ItemsSource as ICollection<FoodModel>;
                if (sourceCollection.Contains(data))
                {
                    sourceCollection.Remove(data);
                }
            }

            if (CursorToSourceVisual.ContainsKey(e.Cursor))
            {
                CursorToSourceVisual.Remove(e.Cursor);
            }
            e.Handled = true;
        }
    }
}