﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Surface.Presentation;
using Microsoft.Surface.Presentation.Controls;
using Blake.NUI.WPF.Utility;
using System.Collections;

namespace SurfaceDragDropSample.Controls
{
    public class CustomItemsControl : ItemsControl
    {
        #region Fields

        Dictionary<SurfaceDragCursor, FrameworkElement> CursorToSourceVisual = new Dictionary<SurfaceDragCursor, FrameworkElement>();

        #endregion

        #region Constructors

        static CustomItemsControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(CustomItemsControl), new FrameworkPropertyMetadata(typeof(CustomItemsControl)));
        }

        public CustomItemsControl()
        {
            Background = Brushes.Transparent;

            this.Loaded += new RoutedEventHandler(CustomItemsControl_Loaded);
            this.Unloaded += new RoutedEventHandler(CustomItemsControl_Unloaded);
        }

        #endregion

        #region Overridden Methods

        protected override void PrepareContainerForItemOverride(DependencyObject element, object item)
        {
            FrameworkElement container = element as FrameworkElement;
            if (container != null)
                container.PreviewTouchDown += container_PreviewTouchDown;
            base.PrepareContainerForItemOverride(element, item);
        }

        protected override void ClearContainerForItemOverride(DependencyObject element, object item)
        {
            FrameworkElement container = element as FrameworkElement;
            if (container != null)
            {
                container.PreviewTouchDown -= container_PreviewTouchDown;
            }

            base.ClearContainerForItemOverride(element, item);
        }

        #endregion

        #region Private Methods

        void CustomItemsControl_Loaded(object sender, RoutedEventArgs e)
        {
            SurfaceDragDrop.AddDropHandler(this, OnCursorDrop);
            SurfaceDragDrop.AddDragCompletedHandler(this, OnDragCompleted);
            SurfaceDragDrop.AddDragCanceledHandler(this, OnDragCanceled);
            this.AllowDrop = true;
        }

        void CustomItemsControl_Unloaded(object sender, RoutedEventArgs e)
        {
            this.AllowDrop = false;
            SurfaceDragDrop.RemoveDropHandler(this, OnCursorDrop);
            SurfaceDragDrop.RemoveDragCompletedHandler(this, OnDragCompleted);
            SurfaceDragDrop.RemoveDragCanceledHandler(this, OnDragCanceled);
        }

        void container_PreviewTouchDown(object sender, TouchEventArgs e)
        {
            FrameworkElement draggedElement = sender as FrameworkElement;

            var data = draggedElement.DataContext;

            List<InputDevice> devices = new List<InputDevice>();
            devices.Add(e.TouchDevice);

            ContentControl cursorVisual = new ContentControl();
            cursorVisual.Content = data;
            cursorVisual.ContentTemplate = this.ItemTemplate;

            SurfaceDragCursor cursor = SurfaceDragDrop.BeginDragDrop(this, draggedElement, cursorVisual, data, devices, DragDropEffects.All);

            cursor.CanRotate = true;
            cursor.CanScale = true;

            draggedElement.Visibility = System.Windows.Visibility.Hidden;
            CursorToSourceVisual.Add(cursor, draggedElement);

            base.OnPreviewTouchDown(e);
        }

        private void OnCursorDrop(object sender, SurfaceDragDropEventArgs e)
        {
            IList targetCollection = ItemsSource as IList;
            if (targetCollection == null)
            {
                targetCollection = Items;
            }

            var data = e.Cursor.Data;
            if (data != null)
                targetCollection.Add(data);
            e.Handled = true;
        }

        private void OnDragCanceled(object sender, SurfaceDragDropEventArgs e)
        {
            if (CursorToSourceVisual.ContainsKey(e.Cursor))
            {
                CursorToSourceVisual[e.Cursor].Visibility = System.Windows.Visibility.Visible;
                CursorToSourceVisual.Remove(e.Cursor);
            }
            e.Handled = true;
        }

        private void OnDragCompleted(object sender, SurfaceDragCompletedEventArgs e)
        {
            var data = e.Cursor.Data as FoodModel;
            if ((e.Cursor.Effects & DragDropEffects.Move) == DragDropEffects.Move)
            {
                IList sourceCollection = ItemsSource as IList;
                if (sourceCollection == null)
                {
                    sourceCollection = Items;
                }
                if (sourceCollection.Contains(data))
                {
                    sourceCollection.Remove(data);
                }
            }

            if (CursorToSourceVisual.ContainsKey(e.Cursor))
            {
                CursorToSourceVisual.Remove(e.Cursor);
            }
            e.Handled = true;
        }

        #endregion
    }
}
