using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;
using Microsoft.Surface.Presentation;
using Microsoft.Surface.Presentation.Controls;
using Microsoft.Surface.Presentation.Input;
using Blake.NUI.WPF.Touch;
using Blake.NUI.WPF.Utility;
using System.Collections.ObjectModel;

namespace SurfaceDragDropSample
{
    /// <summary>
    /// Interaction logic for SurfaceWindow1.xaml
    /// </summary>
    public partial class SurfaceWindow1 : SurfaceWindow
    {
        #region Fields

        Dictionary<SurfaceDragCursor, UIElement> CursorToSourceVisual = new Dictionary<SurfaceDragCursor, UIElement>();

        #endregion

        #region Properties

        #region FruitItems

        private ObservableCollection<FoodModel> _fruitItems;
        public ObservableCollection<FoodModel> FruitItems
        {
            get
            {
                if (_fruitItems == null)
                    _fruitItems = new ObservableCollection<FoodModel>();
                return _fruitItems;
            }
        }

        #endregion

        #region MixedItems

        private ObservableCollection<FoodModel> _mixedItems;
        public ObservableCollection<FoodModel> MixedItems
        {
            get
            {
                if (_mixedItems == null)
                    _mixedItems = new ObservableCollection<FoodModel>();
                return _mixedItems;
            }
        }

        #endregion

        #region VegetableItems

        private ObservableCollection<FoodModel> _vegetableItems;
        public ObservableCollection<FoodModel> VegetableItems
        {
            get
            {
                if (_vegetableItems == null)
                    _vegetableItems = new ObservableCollection<FoodModel>();
                return _vegetableItems;
            }
        }

        #endregion
        
        #region ScatterItems

        private ObservableCollection<FoodModel> _scatterItems;
        public ObservableCollection<FoodModel> ScatterItems
        {
            get
            {
                if (_scatterItems == null)
                    _scatterItems = new ObservableCollection<FoodModel>();
                return _scatterItems;
            }
        }

        #endregion

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor.
        /// </summary>
        public SurfaceWindow1()
        {
            LoadData();
            this.DataContext = this;

            InitializeComponent();
            NativeTouchDevice.RegisterEvents(this);
            MouseTouchDevice.RegisterEvents(this);
        }

        #endregion

        private void LoadData()
        {
            MixedItems.Add(new FoodModel() { Name = "Apple", Type = FoodType.Fruit });
            MixedItems.Add(new FoodModel() { Name = "Broccoli", Type = FoodType.Vegetable });
            MixedItems.Add(new FoodModel() { Name = "Banana", Type = FoodType.Fruit });
            MixedItems.Add(new FoodModel() { Name = "Radish", Type = FoodType.Vegetable });

            FruitItems.Add(new FoodModel() { Name = "Nectarine", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Melon", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Raisin", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Pineapple", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Strawberry", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Raspberry", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Grape", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Cherry", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Blueberry", Type = FoodType.Fruit });
            FruitItems.Add(new FoodModel() { Name = "Kumquat", Type = FoodType.Fruit });
        }

        private void items_DragEnter(object sender, SurfaceDragDropEventArgs e)
        {
            ItemsControl target = sender as ItemsControl;
            var data = e.Cursor.Data as FoodModel;

            if (data == null ||
                e.Cursor.DragSource == target ||
                (data.Type == FoodType.Fruit && target.ItemsSource == VegetableItems) ||
                (data.Type == FoodType.Vegetable && target.ItemsSource == FruitItems))
            {
                e.Effects = DragDropEffects.None;
            }
            e.Handled = true;
        }

        private void items_DragLeave(object sender, SurfaceDragDropEventArgs e)
        {
            e.Effects = e.Cursor.AllowedEffects;
            e.Handled = true;
        }

    }
}