﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace HelloMT
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Border_ManipulationDelta(object sender, ManipulationDeltaEventArgs e)
        {
            //Get who triggered the event
            FrameworkElement element = sender as FrameworkElement;

            //Make sure this is a FrameworkElement
            if (element != null)
            {
                //Initialize a blank Matrix
                Matrix matrix = new Matrix();
                
                //Try to get the existing MatrixTransform
                //The first time, this will return null and we will use the blank Matrix above
                MatrixTransform transformMatrix = element.RenderTransform as MatrixTransform;
                
                //If it exists,
                if (transformMatrix != null)
                {
                    //Then retrieve the matrix from it
                    matrix = transformMatrix.Matrix;
                }
                
                //Apply the delta translation
                matrix.Translate(e.DeltaManipulation.Translation.X, e.DeltaManipulation.Translation.Y);

                //Get ManipulationOrigin relative to the element
                Point transformCenter = LayoutRoot.TranslatePoint(e.ManipulationOrigin, element);
                
                //Then transform it with the current matrix
                transformCenter = matrix.Transform(transformCenter);

                //Apply the delta rotation
                matrix.RotateAt(e.DeltaManipulation.Rotation, transformCenter.X, transformCenter.Y);

                //Apply the delta scale
                matrix.ScaleAt(e.DeltaManipulation.Scale.X, e.DeltaManipulation.Scale.Y, transformCenter.X, transformCenter.Y);

                //Update with the transformed matrix
                element.RenderTransform = new MatrixTransform(matrix);

                //Get a rect of the container
                Rect containerBorder = new Rect(LayoutRoot.RenderSize);

                //Get a transformed rect of the element 
                Rect elementBorder = element.RenderTransform.TransformBounds(new Rect(element.RenderSize));

                //If we are in inertia and the container doesn't fully contain the element
                if (e.IsInertial && !containerBorder.Contains(elementBorder))
                {
                    //Then complete the inertia
                    e.Complete();
                }

                //We took care of the event
                e.Handled = true;
            }
        }

        private void Border_ManipulationStarting(object sender, ManipulationStartingEventArgs e)
        {
            //Manipulations should be relative to the parent element
            e.ManipulationContainer = LayoutRoot;
        }

        private void Border_ManipulationInertiaStarting(object sender, ManipulationInertiaStartingEventArgs e)
        {
            //Smooth deceleration at .001 DIP/millisecond
            e.TranslationBehavior.DesiredDeceleration = .0001;

            //Smooth deceleration at .001 degrees/millisecond
            e.RotationBehavior.DesiredDeceleration = .001;
        }
    }
}
