﻿using System.ComponentModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DataControls
{
    public enum Region
    {
        Northeast,
        South,
        Midwest,
        West
    }

    public class StateModel : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(string info)
        {
            if (PropertyChanged == null)
                return;

            PropertyChanged(this, new PropertyChangedEventArgs(info));
        }

        #endregion

        #region Properties

        private string _stateName;
        public string StateName
        {
            get
            {
                return _stateName;
            }
            set
            {
                _stateName = value;
                NotifyPropertyChanged("StateName");
            }
        }

        private Region _region;
        public Region Region
        {
            get
            {
                return _region;
            }
            set
            {
                _region = value;
                NotifyPropertyChanged("Region");
            }
        }

        private string _flagURIPath;
        public string FlagURIPath
        {
            get
            {
                return _flagURIPath;
            }
            set
            {
                _flagURIPath = value;
                NotifyPropertyChanged("FlagURIPath");
                NotifyPropertyChanged("FlagURI");
            }
        }

        public Uri FlagURI
        {
            get
            {
                return new Uri(FlagURIPath, UriKind.Absolute);
            }
        }
        #endregion

        public StateModel(string name, string flagImagePath, Region region)
        {
            this.StateName = name;
            this.FlagURIPath = flagImagePath;
            this.Region = region;
        }
    }
}