﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Surface.Presentation.Controls;
using System.Windows.Media.Animation;
using Blake.NUI.WPF.SurfaceToolkit.Utility;

namespace DataControls
{
    public partial class StateView : UserControl
    {
        static Stack<Point> availablePositions = new Stack<Point>();

        static StateView()
        {
            //HACK: just a quick way to create a grid of positions
            for (int i = 1275; i >= 75; i -= 150)
            {
                for (int j = 675; j >= 75; j -= 100)
                {
                    availablePositions.Push(new Point(i, j));
                }
            }
        }

        public StateView()
        {
            InitializeComponent();
        }

        public StateView(StateModel model)
        {
            InitializeComponent();
            this.LayoutRoot.DataContext = model;
        }

        private void control_Loaded(object sender, RoutedEventArgs e)
        {
            ConfigureSVI();
        }

        void ConfigureSVI()
        {
            ScatterView sv = FindVisualParent<ScatterView>(this);

            if (sv == null)
                return;
            ScatterViewItem svi = sv.ItemContainerGenerator.ContainerFromItem(this.DataContext) as ScatterViewItem;

            if (svi == null)
                return;

            double aspectRatio = this.viewbox.ActualWidth / this.viewbox.ActualHeight;

            svi.Height = 100;
            svi.Width = svi.Height * aspectRatio;

            svi.Resources.Add("SVIShadowRadius", 20.0);

            svi.ContainerActivated += (s, e) =>
            {
                VisualStateManager.GoToState(this, "Active", true);
            };

            svi.ContainerDeactivated += (s, e) =>
            {
                VisualStateManager.GoToState(this, "Inactive", true);
            };


            Point targetPoint = new Point(100, 100);
            if (availablePositions.Count > 0)
                targetPoint = availablePositions.Pop();

            svi.Orientation = -90;
            svi.Center = new Point(targetPoint.X, -300);

            BackEase backease = new BackEase();
            backease.EasingMode = EasingMode.EaseOut;
            backease.Amplitude = 0.3;

            double startTime = 1 + (700 - targetPoint.Y) / 700 + targetPoint.X / 2000;
            SurfaceAnimateUtility.ThrowSVI(svi, targetPoint, 0.0, startTime, startTime + 1, backease);

        }

        static T FindVisualParent<T>(DependencyObject obj) where T : UIElement
        {
            if (obj is T)
            {
                return obj as T;
            }

            DependencyObject parent = VisualTreeHelper.GetParent(obj);
            if (parent == null)
                return null;
            return FindVisualParent<T>(parent);
        }
    }
}
