﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Surface.Presentation.Controls;

namespace DataControls
{
    public partial class StateView : UserControl
    {
        bool isFront = true;

        public StateView()
        {
            InitializeComponent();
        }

        public StateView(StateModel model)
        {
            InitializeComponent();
            this.LayoutRoot.DataContext = model;
        }

        private void control_Loaded(object sender, RoutedEventArgs e)
        {
            ConfigureSVI();
        }

        void ConfigureSVI()
        {
            ScatterView sv = FindVisualParent<ScatterView>(this);

            if (sv == null)
                return;
            ScatterViewItem svi = sv.ItemContainerGenerator.ContainerFromItem(this.DataContext) as ScatterViewItem;

            if (svi == null)
                return;
            this.image.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
            double aspectRatio = this.image.DesiredSize.Width / this.image.DesiredSize.Height;

            double height = this.image.DesiredSize.Height;
            svi.Height = height;
            svi.Width = height * aspectRatio;
            this.plane.Height = height;
            this.plane.Width = height * aspectRatio;

            svi.Orientation = 0;
            shadowFront.DataContext = svi;
            shadowBack.DataContext = svi;

            svi.MaxHeight = 800;
            svi.MaxWidth = svi.MaxHeight * aspectRatio;
            svi.MinHeight = 150;
            svi.MinWidth = svi.MinHeight * aspectRatio;
            
            svi.ContainerActivated += (s, e) =>
            {
                VisualStateManager.GoToState(this, "Active", true);
            };

            svi.ContainerDeactivated += (s, e) =>
            {
                VisualStateManager.GoToState(this, "Inactive", true);
            };

        }

        static T FindVisualParent<T>(DependencyObject obj) where T : UIElement
        {
            if (obj is T)
            {
                return obj as T;
            }

            DependencyObject parent = VisualTreeHelper.GetParent(obj);
            if (parent == null)
                return null;
            return FindVisualParent<T>(parent);
        }

        private void Grid_TouchDown(object sender, TouchEventArgs e)
        {
            if (isFront)
            {
                VisualStateManager.GoToState(this, "Back", true);
                isFront = false;
            }
            else
            {
                VisualStateManager.GoToState(this, "Front", true);
                isFront = true;
            }
        }
    }
}
