﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Windows.Shapes;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Documents;

namespace CustomTouchVisualizations
{
    public class TouchVisualizationAdorner : Adorner
    {
        #region Class members

        VisualCollection visualChildren;

        Dictionary<TouchDevice, Ellipse> auras = new Dictionary<TouchDevice, Ellipse>();
        UIElement auraElement;
        Canvas auraCanvas;

        #endregion

        #region Static methods

        public static void AddAuras(UIElement auraElement)
        {
            AdornerLayer layer = AdornerLayer.GetAdornerLayer(auraElement);
            if (layer == null)
                throw new NullReferenceException("Adorner layer is null");
            TouchVisualizationAdorner auras = new TouchVisualizationAdorner(auraElement);
            layer.Add(auras);
        }

        #endregion

        #region Constructors

        public TouchVisualizationAdorner(UIElement auraElement) : base(auraElement)
        {
            visualChildren = new VisualCollection(this);

            this.auraElement = auraElement;
            if (this.auraElement == null)
                throw new ArgumentNullException("auraElement");

            this.auraCanvas = new Canvas();
            this.IsHitTestVisible = false;

            visualChildren.Add(this.auraCanvas);

            this.auraElement.AddHandler(UIElement.TouchEnterEvent, new EventHandler<TouchEventArgs>(TouchEvent), true);
            this.auraElement.AddHandler(UIElement.PreviewTouchMoveEvent, new EventHandler<TouchEventArgs>(TouchEvent), true);
            this.auraElement.AddHandler(UIElement.TouchLeaveEvent, new EventHandler<TouchEventArgs>(TouchEvent), true);
        }

        #endregion

        #region Overridden methods

        protected override int VisualChildrenCount
        {
            get
            {
                return visualChildren.Count;
            }
        }

        protected override Visual GetVisualChild(int index)
        {
            return visualChildren[index];
        }

        protected override Size MeasureOverride(Size constraint)
        {
            this.auraCanvas.Measure(constraint);
            return this.auraCanvas.DesiredSize;
        }

        protected override Size ArrangeOverride(Size finalSize)
        {
            this.auraCanvas.Arrange(new Rect(finalSize));
            return finalSize;
        }

        #endregion

        #region Touch Events

        private void TouchEvent(object sender, TouchEventArgs e)
        {
            VisualizeTouches(e);
        }

        #endregion

        #region Visualize Touches

        private void VisualizeTouches(TouchEventArgs e)
        {
            Ellipse ellipse;
            bool auraExists = auras.TryGetValue(e.TouchDevice, out ellipse);

            TouchPoint touch = e.GetTouchPoint(auraCanvas);

            if (auraExists &&
                (touch.Action == TouchAction.Up ||
                 e.TouchDevice.Target == null))
            {
                if (auraCanvas.Children.Contains(ellipse))
                {
                    auraCanvas.Children.Remove(ellipse);
                }
                auras.Remove(e.TouchDevice);
            }
            else
            {
                if (!auraExists)
                {
                    ellipse = new Ellipse();
                    ellipse.Fill = new RadialGradientBrush(Colors.Gray, Colors.Transparent);

                    auraCanvas.Children.Add(ellipse);
                    auras.Add(e.TouchDevice, ellipse);
                }

                ellipse.Width = touch.Size.Width * 2;
                ellipse.Height = touch.Size.Height * 2;

                Canvas.SetLeft(ellipse, touch.Position.X - ellipse.Width / 2);
                Canvas.SetTop(ellipse, touch.Position.Y - ellipse.Height / 2);
            }
        }

        #endregion
    }
}
