﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity;
using System.Windows;
using System.Windows.Input;

namespace TouchGestureLibrary
{
    public class DoubleTapGestureTrigger : TriggerBase<UIElement>
    {
        #region Class members

        DoubleTapGestureEngine gestureEngine;

        #endregion

        #region Properties

        private int _minMilliseconds = 0;
        public int MinMilliseconds
        {
            get
            {
                return _minMilliseconds;
            }
            set
            {
                _minMilliseconds = value;
            }
        }
        
        private int _maxMilliseconds = 1000;
        public int MaxMilliseconds
        {
            get
            {
                return _maxMilliseconds;
            }
            set
            {
                _maxMilliseconds = value;
            }
        }

        private int _gapMilliseconds = 400;
        public int GapMilliseconds
        {
            get
            {
                return _gapMilliseconds;
            }
            set
            {
                _gapMilliseconds = value;
            }
        }

        private double _maxMovement = 25;
        public double MaxMovement
        {
            get
            {
                return _maxMovement;
            }
            set
            {
                _maxMovement = value;
            }
        }

        #endregion
        
        #region Events

        public event EventHandler DoubleTap;

        private void OnDoubleTap()
        {
            this.InvokeActions(null);
            if (DoubleTap != null)
            {
                DoubleTap(this.AssociatedObject, EventArgs.Empty);
            }
        }

        #endregion

        #region Constructors

        public DoubleTapGestureTrigger()
            : base()
        {
            
        }

        #endregion

        #region Overridden methods

        protected override void OnAttached()
        {
            base.OnAttached();
            this.AssociatedObject.TouchDown += new EventHandler<TouchEventArgs>(AssociatedObject_TouchDown);
            this.AssociatedObject.TouchMove += new EventHandler<TouchEventArgs>(AssociatedObject_TouchMove);
            this.AssociatedObject.TouchUp += new EventHandler<TouchEventArgs>(AssociatedObject_TouchUp);
        }
        
        #endregion

        #region Event handlers
        
        void AssociatedObject_TouchDown(object sender, TouchEventArgs e)
        {
            //e.Handled = true;
            if (gestureEngine == null ||
                gestureEngine.IsAborted ||
                gestureEngine.IsCompleted)
            {

                gestureEngine = new DoubleTapGestureEngine(MinMilliseconds, GapMilliseconds, MaxMilliseconds, MaxMovement);
                gestureEngine.GestureAborted += new EventHandler(gesture_GestureAborted);
                gestureEngine.GestureStarted += new EventHandler(gesture_GestureStarted);
                gestureEngine.GestureCompleted += new EventHandler(gesture_GestureCompleted);
            }
            DateTime dt = TimestampToDateTime(e.Timestamp);

            Point position = e.GetTouchPoint(AssociatedObject).Position;

            gestureEngine.TrackTouchDown(dt, position);
        }

        void AssociatedObject_TouchMove(object sender, TouchEventArgs e)
        {
            //e.Handled = true;
            
            DateTime dt = TimestampToDateTime(e.Timestamp);

            Point position = e.GetTouchPoint(AssociatedObject).Position;

            if (gestureEngine != null)
                gestureEngine.TrackTouchMove(dt, position);
        }

        void AssociatedObject_TouchUp(object sender, TouchEventArgs e)
        {
            //e.Handled = true;
            
            DateTime dt = TimestampToDateTime(e.Timestamp);

            Point position = e.GetTouchPoint(AssociatedObject).Position;

            if (gestureEngine != null)
                gestureEngine.TrackTouchUp(dt, position);
        }

        private static DateTime TimestampToDateTime(int timestamp)
        {
            DateTime dt = DateTime.Now;
            dt.AddMilliseconds(Environment.TickCount - timestamp);
            return dt;
        }
                
        #endregion

        #region Gesture events

        void gesture_GestureCompleted(object sender, EventArgs e)
        {
            OnDoubleTap();
        }

        void gesture_GestureStarted(object sender, EventArgs e)
        {     
        }

        void gesture_GestureAborted(object sender, EventArgs e)
        {
        }

        #endregion
    }
}
