﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Windows.Threading;
using System.Windows.Interop;
using System.ComponentModel;
using System.Threading;
using System.Runtime.InteropServices;
using Blake.NUI.WPF.Touch;
using System.Reflection;
using Blake.NUI.WPF.Touch.Interop;

namespace TouchTraces
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Dictionary<TouchDevice, TouchPointData> touches = new Dictionary<TouchDevice, TouchPointData>();
        Dictionary<TouchDevice, Point> lastPoints = new Dictionary<TouchDevice, Point>();
        
        Dictionary<StylusDevice, Point> lastEvents = new Dictionary<StylusDevice, Point>();
        Dictionary<StylusDevice, DateTime> lastEventTimes = new Dictionary<StylusDevice, DateTime>();

        DispatcherTimer timer;
        int numEvents = 0;
        int numEvents32 = 0;
        DateTime lastReset;

        DateTime appstart = DateTime.Now;

        List<Brush> colors = new List<Brush>();
        int currentColor = 0;

        public MainWindow()
        {
            InitializeComponent();

            QuickTouchDevice.RegisterEvents(this);
            timer = new DispatcherTimer();
            timer.Tick += new EventHandler(timer_Tick);
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Start();
            lastReset = DateTime.Now;

            colors.Add(new SolidColorBrush(Colors.Black));
            colors.Add(new SolidColorBrush(Colors.Blue));
            colors.Add(new SolidColorBrush(Colors.Purple));
            colors.Add(new SolidColorBrush(Colors.Red));
            colors.Add(new SolidColorBrush(Colors.Orange));
            colors.Add(new SolidColorBrush(Colors.Cyan));
            colors.Add(new SolidColorBrush(Colors.Yellow));
            colors.Add(new SolidColorBrush(Colors.HotPink));

            colors.ForEach(c => c.Freeze());
        }

        #region Standard TouchTraces

        void timer_Tick(object sender, EventArgs e)
        {
            TimeSpan span = DateTime.Now - lastReset;

            txtUpdateSeconds.Text = span.TotalSeconds.ToString("0.00");
            txtTime.Text = (DateTime.Now - appstart).TotalSeconds.ToString("0.00");
            double rate = numEvents / span.TotalSeconds;

            txtEventRate.Text = rate.ToString("0.0");

            rate = numEvents32 / span.TotalSeconds;
            txtEventRate32.Text = rate.ToString("0.0");

            numEvents = 0;
            numEvents32 = 0;
            lastReset = DateTime.Now;
        }

        private void UpdateTouchData(TouchEventArgs e)
        {
            TouchPointData data;

            if (!touches.TryGetValue(e.TouchDevice, out data))
            {
                data = new TouchPointData();
                data.Brush = GetNextColor();
                touches.Add(e.TouchDevice, data);
                mainCanvas.Children.Add(data);
            }
            TouchPoint point = e.GetTouchPoint(mainCanvas);
            data.TouchPoint = point;
            Size size = point.Size;
            Canvas.SetLeft(data, point.Position.X - size.Width / 2);
            Canvas.SetTop(data, point.Position.Y - size.Height / 2);
        }

        void UpdateNum()
        {
            txtNumTouches.Text = this.TouchesOver.Count().ToString();
        }

        private void mainCanvas_TouchDown(object sender, TouchEventArgs e)
        {
            mainCanvas.CaptureTouch(e.TouchDevice);
            UpdateTouchData(e);
            lastPoints[e.TouchDevice] = e.GetTouchPoint(null).Position;
            e.Handled = true;
        }

        private Brush GetNextColor()
        {
            currentColor++;
            if (currentColor > colors.Count - 1)
                currentColor = 0;
            return colors[currentColor];
        }

        private void mainCanvas_TouchMove(object sender, TouchEventArgs e)
        {
            e.Handled = true;

            //TouchPointCollection collection = e.GetIntermediateTouchPoints(mainCanvas);
            //numEvents += collection.Count;
            numEvents += 1;

            TimeSpan span = DateTime.Now - lastReset;

            if (span.TotalSeconds > 1.5)
            {
                txtUpdateSeconds.Text = span.TotalSeconds.ToString("0.00");
            }

            UpdateTouchData(e);

            Point point = e.GetTouchPoint(null).Position;

            if (lastPoints.ContainsKey(e.TouchDevice))
            {
                Point lastPoint = lastPoints[e.TouchDevice];

                Line line = new Line();
                line.Stroke = touches[e.TouchDevice].Brush;
                line.StrokeThickness = 1;
                line.X1 = lastPoint.X;
                line.Y1 = lastPoint.Y;
                line.X2 = point.X;
                line.Y2 = point.Y;
                line.Tag = e.TouchDevice;

                mainCanvas.Children.Add(line);
            }
            lastPoints[e.TouchDevice] = point;
            
        }

        private void mainCanvas_TouchUp(object sender, TouchEventArgs e)
        {
            e.Handled = true;
            TouchPointData data;

            if (touches.TryGetValue(e.TouchDevice, out data))
            {
                touches.Remove(e.TouchDevice);

                if (mainCanvas.Children.Contains(data))
                {
                    mainCanvas.Children.Remove(data);
                }
            }

            List<Line> linesToDelete = new List<Line>();

            foreach (UIElement element in mainCanvas.Children)
            {
                Line line = element as Line;
                if (line != null &&
                    line.Tag == e.TouchDevice)
                {
                    linesToDelete.Add(line);
                }
            }

            foreach (Line line in linesToDelete)
            {
                mainCanvas.Children.Remove(line);
            }
        }

        private void Button_TouchDown(object sender, TouchEventArgs e)
        {
            TouchPointCollection points = e.GetIntermediateTouchPoints(this);

        }


        private void Grid_TouchEnter(object sender, TouchEventArgs e)
        {

            UpdateNum();
        }

        private void Grid_TouchLeave(object sender, TouchEventArgs e)
        {

            UpdateNum();
        }
        #endregion

        private void mainCanvas_StylusDown(object sender, StylusDownEventArgs e)
        {

        }

        private void mainCanvas_MouseDown(object sender, MouseButtonEventArgs e)
        {

        }

    }
}
