﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace TouchTraces
{
    /// <summary>
    /// Interaction logic for TouchPointData.xaml
    /// </summary>
    public partial class TouchPointData : UserControl, INotifyPropertyChanged
    {

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void RaisePropertyChanged(String info)
        {
            if (PropertyChanged == null)
                return;

            PropertyChanged(this, new PropertyChangedEventArgs(info));
        }

        #endregion

        #region Properties

        #region Action

        /// <summary>
        /// The <see cref="Action" /> property's name.
        /// </summary>
        public const string ActionPropertyName = "Action";

        private TouchAction _action = TouchAction.Down;

        /// <summary>
        /// Gets the Action property.
        /// </summary>
        public TouchAction Action
        {
            get
            {
                return _action;
            }

            set
            {
                if (_action == value)
                {
                    return;
                }

                var oldValue = _action;
                _action = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(ActionPropertyName);
            }
        }

        #endregion

        #region Position

        /// <summary>
        /// The <see cref="Position" /> property's name.
        /// </summary>
        public const string PositionPropertyName = "Position";

        private Point _position = new Point(0, 0);

        /// <summary>
        /// Gets the Position property.
        /// </summary>
        public Point Position
        {
            get
            {
                return _position;
            }

            set
            {
                if (_position == value)
                {
                    return;
                }

                var oldValue = _position;
                _position = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(PositionPropertyName);
            }
        }

        #endregion

        #region Size

        /// <summary>
        /// The <see cref="Size" /> property's name.
        /// </summary>
        public const string SizePropertyName = "Size";

        private Size _size = new Size(1, 1);

        /// <summary>
        /// Gets the Size property.
        /// </summary>
        public Size Size
        {
            get
            {
                return _size;
            }

            set
            {
                if (_size == value)
                {
                    return;
                }

                var oldValue = _size;
                _size = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(SizePropertyName);
            }
        }
        #endregion

        #region Id

        /// <summary>
        /// The <see cref="Id" /> property's name.
        /// </summary>
        public const string IdPropertyName = "Id";

        private int _id = 0;

        /// <summary>
        /// Gets the Id property.
        /// </summary>
        public int Id
        {
            get
            {
                return _id;
            }

            set
            {
                if (_id == value)
                {
                    return;
                }

                var oldValue = _id;
                _id = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(IdPropertyName);
            }
        }
        #endregion

        #region TouchPoint

        /// <summary>
        /// The <see cref="TouchPoint" /> property's name.
        /// </summary>
        public const string TouchPointPropertyName = "TouchPoint";

        private TouchPoint _touchPoint = null;

        /// <summary>
        /// Gets the TouchPoint property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public TouchPoint TouchPoint
        {
            get
            {
                return _touchPoint;
            }

            set
            {
                if (_touchPoint == value)
                {
                    return;
                }

                var oldValue = _touchPoint;
                _touchPoint = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(TouchPointPropertyName);

                UpdateProperties();
            }
        }

        private void UpdateProperties()
        {
            if (TouchPoint == null)
                return;

            this.Action = TouchPoint.Action;
            this.Position = TouchPoint.Position;
            this.Size = TouchPoint.Size;
            this.Id = TouchPoint.TouchDevice.Id;
        }

        #endregion
        
        #region Brush

        public Brush Brush { get; set; }
        
        #endregion
        
        #endregion

        public TouchPointData()
        {
            this.DataContext = this;
            InitializeComponent();
        }
    }
}
