﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Windows.Threading;

namespace TouchTraces
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Dictionary<TouchDevice, TouchPointData> touches = new Dictionary<TouchDevice, TouchPointData>();
        Dictionary<TouchDevice, Point> lastPoints = new Dictionary<TouchDevice, Point>();

        DispatcherTimer timer;
        int numEvents = 0;
        DateTime lastReset;

        public MainWindow()
        {
            InitializeComponent();

            timer = new DispatcherTimer();
            timer.Tick += new EventHandler(timer_Tick);
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Start();
            lastReset = DateTime.Now;
        }

        void timer_Tick(object sender, EventArgs e)
        {
            TimeSpan span = DateTime.Now - lastReset;

            txtUpdateSeconds.Text = span.TotalSeconds.ToString("0.00");

            double rate = numEvents / span.TotalSeconds;
            txtEventRate.Text = rate.ToString("0.0");
            numEvents = 0;
            lastReset = DateTime.Now;
        }

        private void UpdateTouchData(TouchEventArgs e)
        {
            TouchPointData data;

            if (!touches.TryGetValue(e.TouchDevice, out data))
            {
                data = new TouchPointData();
                touches.Add(e.TouchDevice, data);
                mainCanvas.Children.Add(data);
            }

            data.TouchPoint = e.GetTouchPoint(null);

            Canvas.SetLeft(data, data.TouchPoint.Position.X - data.TouchPoint.Size.Width / 2);
            Canvas.SetTop(data, data.TouchPoint.Position.Y - data.TouchPoint.Size.Height / 2);
        }

        private void mainCanvas_TouchDown(object sender, TouchEventArgs e)
        {
            UpdateTouchData(e);

            lastPoints[e.TouchDevice] = e.GetTouchPoint(null).Position;
        }

        private void mainCanvas_TouchMove(object sender, TouchEventArgs e)
        {
            //TouchPointCollection collection = e.GetIntermediateTouchPoints(mainCanvas);
            //numEvents += collection.Count;
            numEvents += 1;

            TimeSpan span = DateTime.Now - lastReset;

            if (span.TotalSeconds > 1.5)
            {
                txtUpdateSeconds.Text = span.TotalSeconds.ToString("0.00");
            }

            UpdateTouchData(e);
            /*
            Point point = e.GetTouchPoint(null).Position;
            Point lastPoint = lastPoints[e.TouchDevice];
            
            Line line = new Line();
            line.Stroke = new SolidColorBrush(Colors.Black);
            line.StrokeThickness = 1;
            line.X1 = lastPoint.X;
            line.Y1 = lastPoint.Y;
            line.X2 = point.X;
            line.Y2 = point.Y;
            line.Tag = e.TouchDevice;

            mainCanvas.Children.Add(line);
            
            lastPoints[e.TouchDevice] = point;
             */
        }

        private void mainCanvas_TouchUp(object sender, TouchEventArgs e)
        {
            TouchPointData data;

            if (touches.TryGetValue(e.TouchDevice, out data))
            {
                touches.Remove(e.TouchDevice);

                if (mainCanvas.Children.Contains(data))
                {
                    mainCanvas.Children.Remove(data);
                }
            }

            List<Line> linesToDelete = new List<Line>();

            foreach (UIElement element in mainCanvas.Children)
            {
                Line line = element as Line;
                if (line != null &&
                    line.Tag == e.TouchDevice)
                {
                    linesToDelete.Add(line);
                }
            }

            foreach (Line line in linesToDelete)
            {
                mainCanvas.Children.Remove(line);
            }
        }

        private void Button_TouchDown(object sender, TouchEventArgs e)
        {
            TouchPointCollection points = e.GetIntermediateTouchPoints(this);
            
        }
    }
}
