﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Blake.NUI.WPF.Utility;
using Blake.NUI.WPF.ManipulationHelpers;
using Microsoft.Surface.Presentation.Input;

namespace BoundarySlider
{
    public partial class Boundary1DSlider : UserControl
    {
        BoundedManipulationRange horizontalRange;

        public Boundary1DSlider()
        {
            InitializeComponent();

            horizontalRange = new BoundedManipulationRange()
                {
                    ElasticMargin = 40,
                    Position = thumbTranslate.X
                };
        }

        private void thumb_ManipulationStarting(object sender, ManipulationStartingEventArgs e)
        {
            e.ManipulationContainer = container;
            e.Handled = true;
        }

        private void thumb_ManipulationDelta(object sender, ManipulationDeltaEventArgs e)
        {
            UpdateRanges();

            horizontalRange.Position += e.DeltaManipulation.Translation.X;
            thumbTranslate.X = horizontalRange.BoundedPosition;

            if (e.IsInertial)
            {
                //During inertia trying to push past elasticMargin
                if (horizontalRange.IsPositionBelowLowerElasticMargin &&
                    e.Velocities.LinearVelocity.X < 0)
                {
                    //Restart inertia so we can bounce back immediately instead of waiting
                    e.StartInertia();
                }
                else if (horizontalRange.IsPositionAboveUpperElasticMargin &&
                            e.Velocities.LinearVelocity.X > 0)
                {
                    e.StartInertia();
                }
            }

            e.ReportBoundaryFeedback(new ManipulationDelta(new Vector(horizontalRange.ElasticOffset,
                                                                      0),
                                                           0,
                                                           new Vector(),
                                                           new Vector()));

            e.Handled = true;
        }

        private void thumb_ManipulationBoundaryFeedback(object sender, ManipulationBoundaryFeedbackEventArgs e)
        {
            Vector delta = e.BoundaryFeedback.Translation;

            containerTranslate.X = delta.X;
            containerTranslate.Y = delta.Y;

            e.Handled = true;
        }

        private void thumb_ManipulationInertiaStarting(object sender, ManipulationInertiaStartingEventArgs e)
        {
            if (horizontalRange.IsPositionOutOfBounds)
            {
                //if we're out of bounds
                double displacement = Math.Abs(horizontalRange.BoundaryOverflow) + 1;
                e.TranslationBehavior.DesiredDisplacement = displacement;

                Vector velocity = new Vector();
                velocity.X = -Math.Min(2, displacement / 10) * Math.Sign(horizontalRange.BoundaryOverflow);
                e.TranslationBehavior.InitialVelocity = velocity;
            }
            else
            {
                //otherwise regular inertia within the range
                e.TranslationBehavior.DesiredDeceleration = 0.002;
            }
            e.Handled = true;
        }

        private void thumb_ManipulationCompleted(object sender, ManipulationCompletedEventArgs e)
        {
            UpdateRanges();

            if (horizontalRange.IsPositionOutOfBounds)
            {
                //Manipulation has completed but position is beyond unstretch range.
                //This can happen when inertia completes after flicking the object past the boundary.
                //Use this helper to restart the manipulation so inertia logic can take over.
                InertiaDummyManipulator.ManipulateAndStartInertia(thumb);
            }
            e.Handled = true;
        }

        private void UpdateRanges()
        {
            horizontalRange.LowerBoundary = 0;
            horizontalRange.UpperBoundary = container.ActualWidth - thumb.ActualWidth;
        }
    }
}
