﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SlideOutControlTest.Controls
{
    public class SlideOutControl : HeaderedContentControl
    {
        #region Properties

        private double _controlPosition = 0;
        private double ControlPosition
        {
            get
            {
                return _controlPosition;
            }
            set
            {
                _controlPosition = Math.Max(0, Math.Min(value, ContentSize.Height));
            }
        }

        protected TranslateTransform _translateTransform = new TranslateTransform();
        protected TranslateTransform TranslateTransform
        {
            get
            {
                return _translateTransform;
            }
        }

        protected Size HeaderSize { get; set; }
        protected Size ContentSize { get; set; }

        protected Size TotalSize
        {
            get
            {
                return new Size(Math.Max(HeaderSize.Width, ContentSize.Width),
                                HeaderSize.Height + ContentSize.Height);
            }
        }

        #endregion

        #region Template Parts

        #region RectClip

        private const string RectClip_Name = "PART_RectClip";

        private RectangleGeometry _rectClip = null;

        protected RectangleGeometry RectClip
        {
            get
            {
                return _rectClip;
            }

            set
            {
                _rectClip = value;
            }
        }

        #endregion

        #region TouchTarget

        private const string TouchTarget_Name = "PART_TouchTarget";
        
        private FrameworkElement _touchTarget = null;
        protected FrameworkElement TouchTarget
        {
            get
            {
                return _touchTarget;
            }
            set
            {
                if (_touchTarget == value)
                    return;

                if (_touchTarget != null)
                {
                    _touchTarget.ManipulationStarting -= _touchTarget_ManipulationStarting;
                    _touchTarget.ManipulationDelta -= _touchTarget_ManipulationDelta;
                }

                _touchTarget = value;

                if (_touchTarget != null)
                {
                    _touchTarget.IsManipulationEnabled = true;
                    _touchTarget.ManipulationStarting += _touchTarget_ManipulationStarting;
                    _touchTarget.ManipulationDelta += _touchTarget_ManipulationDelta;
                    Canvas.SetTop(_touchTarget, 0);
                    Canvas.SetLeft(_touchTarget, 0);
                    _touchTarget.RenderTransform = TranslateTransform;
                }
            }
        }

        #endregion

        #region HeaderContainer

        private const string HeaderContainer_Name = "PART_HeaderContainer";

        private FrameworkElement _headerContainer = null;

        protected FrameworkElement HeaderContainer
        {
            get
            {
                return _headerContainer;
            }
            set
            {
                if (_headerContainer == value)
                    return;

                if (_headerContainer != null)
                {

                }

                _headerContainer = value;

                if (_headerContainer != null)
                {

                }
            }
        }

        #endregion

        #region ContentContainer

        private const string ContentContainer_Name = "PART_ContentContainer";

        private FrameworkElement _contentContainer = null;

        protected FrameworkElement ContentContainer
        {
            get
            {
                return _contentContainer;
            }
            set
            {
                if (_contentContainer == value)
                    return;

                if (_contentContainer != null)
                {

                }

                _contentContainer = value;

                if (_contentContainer != null)
                {
                }
            }
        }

        #endregion

        #endregion
        
        #region Constructors

        static SlideOutControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SlideOutControl), new FrameworkPropertyMetadata(typeof(SlideOutControl)));
        }

        public SlideOutControl()
        {
            this.Loaded += new RoutedEventHandler(SlideOutControl_Loaded);
        }
        
        #endregion

        #region Overridden Methods

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            TouchTarget = this.GetTemplateChild(TouchTarget_Name) as FrameworkElement;
            HeaderContainer = this.GetTemplateChild(HeaderContainer_Name) as FrameworkElement;
            ContentContainer = this.GetTemplateChild(ContentContainer_Name) as FrameworkElement;
            RectClip = this.GetTemplateChild(RectClip_Name) as RectangleGeometry;
        }
        
        #endregion

        #region Private Methods

        void SlideOutControl_Loaded(object sender, RoutedEventArgs e)
        {
            MeasureContent();
            ControlPosition = 0;
            UpdateClip();
        }

        void UpdateClip()
        {
            Size totalSize = this.TotalSize;

            double controlHeight = HeaderSize.Height + ControlPosition;
            RectClip.Rect = new Rect(0, totalSize.Height - controlHeight, 
                                     totalSize.Width, controlHeight);

            TranslateTransform.Y = ControlPosition - ContentSize.Height;
        }

        private void MeasureContent()
        {
            this.HeaderSize = GetElementSize(this.HeaderContainer);
            this.ContentSize = GetElementSize(this.ContentContainer);
        }

        private static Size GetElementSize(FrameworkElement element)
        {
            if (element == null)
                return Size.Empty;

            if (!element.IsMeasureValid)
                element.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));

            return element.DesiredSize;
        }

        #region TouchTarget Manipulation Event Handlers
        
        void _touchTarget_ManipulationStarting(object sender, ManipulationStartingEventArgs e)
        {
            if (e.OriginalSource != this.TouchTarget)
                return;
            e.ManipulationContainer = this;
            e.Mode = ManipulationModes.Translate;
            e.Handled = true;
        }
        
        void _touchTarget_ManipulationDelta(object sender, ManipulationDeltaEventArgs e)
        {
            if (e.OriginalSource != this.TouchTarget)
                return;
            e.Handled = true;
            
            ControlPosition += e.DeltaManipulation.Translation.Y;

            UpdateClip();
        }
        
        #endregion

        #endregion

    }
}
